# -*- coding: utf-8 -*-

import signal
from itertools import groupby, islice

try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest

def per_section(it, is_delimiter=lambda x: x.isspace()):
    """
    From http://stackoverflow.com/a/25226944/610569
    """
    ret = []
    for line in it:
        if is_delimiter(line):
            if ret:
                yield ret  # OR  ''.join(ret)
                ret = []
        else:
            ret.append(line.rstrip())  # OR  ret.append(line)
    if ret:
        yield ret

def per_chunk(iterable, n=1, fillvalue=None):
    """
    From http://stackoverflow.com/a/8991553/610569

        >>> list(per_chunk('abcdefghi', n=2))
        [('a', 'b'), ('c', 'd'), ('e', 'f'), ('g', 'h'), ('i', None)]
        >>> list(per_chunk('abcdefghi', n=3))
        [('a', 'b', 'c'), ('d', 'e', 'f'), ('g', 'h', 'i')]
    """
    args = [iter(iterable)] * n
    return zip_longest(*args, fillvalue=fillvalue)

def per_window(sequence, n=1):
    """
    From http://stackoverflow.com/q/42220614/610569

        >>> list(per_window([1,2,3,4], n=2))
        [(1, 2), (2, 3), (3, 4)]
        >>> list(per_window([1,2,3,4], n=3))
        [(1, 2, 3), (2, 3, 4)]
    """
    start, stop = 0, n
    seq = list(sequence)
    while stop <= len(seq):
        yield tuple(seq[start:stop])
        start += 1
        stop += 1

def zigzag(sequence):
    """
    http://stackoverflow.com/a/1442794/610569

        >>> zigzag(list(range(10)))
        ([0, 2, 4, 6, 8], [1, 3, 5, 7, 9])
    """
    return sequence[::2], sequence[1::2] # Returns (evens, odds)

def skipping_window(sequence, target, n=3):
    """
    Return a sliding window with a constraint to check that
    target is inside the window.
    From http://stackoverflow.com/q/43626525/610569

        >>> list(skipping_window([1,2,3,4,5], 2, 3))
        [(1, 2, 3), (2, 3, 4)]
    """
    start, stop = 0, n
    seq = list(sequence)
    while stop <= len(seq):
        subseq = seq[start:stop]
        if target in subseq:
            yield tuple(seq[start:stop])
        start += 1
        stop += 1
        # Fast forwarding the start.
        # Find the next window which contains the target.
        try:
            # `seq.index(target, start) - (n-1)` would be the next
            # window where the constraint is met.
            start = max(seq.index(target, start) - (n-1), start)
            stop = start + n
        except ValueError:
            break

def camel_shuffle(sequence):
    """
    Inspired by https://stackoverflow.com/q/42549212/610569

        >>> list(range(12)) # Linear.
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

        >>> camel_shuffle(list(range(12))) # M-shape.
        [0, 4, 8, 9, 5, 1, 2, 6, 10, 11, 7, 3]

        >>> camel_shuffle(list(reversed(range(12)))) #W-shape.
        [11, 7, 3, 2, 6, 10, 9, 5, 1, 0, 4, 8]
    """
    one_three, two_four = zigzag(sequence)
    one, three = zigzag(one_three)
    two, four = zigzag(two_four)
    return one + list(reversed(two)) + three + list(reversed(four))


class timeout:
    """
    From https://stackoverflow.com/a/22348885/610569 , e.g. 
    
        >>> with timeout(seconds=3):
        ...    time.sleep(4)
    
    Note: This does not work if the function is catching generic exception, e.g.
        def bad_func(): 
            try: time.sleep(100); 
            except Exception: pass;
        
        wih timeout(seconds=3):
            bad_func()
    """
    def __init__(self, seconds=1, error_message='Timeout'):
        self.seconds = seconds
        self.error_message = error_message
    def handle_timeout(self, signum, frame):
        raise TimeoutError(self.error_message)
    def __enter__(self):
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.seconds)
    def __exit__(self, type, value, traceback):
        signal.alarm(0)



# -*- coding: utf-8 -*-

from lazyme import per_section

with open('10sents.soft') as fin:
    for sentence_attention in per_section(fin):
        # Reading the headerline
        headers = sentence_attention[0]
        sent_id, trg_str, score, src_str, num_src, num_trg = headers
        sent_id, score, num_src, num_trg = int(sent_id), float(score), int(num_src), int(num_trg)

        trg_str = trg_str.split()
        src_str = src_str.split()



# -*- coding: utf-8 -*-

import os

try: # Try importing Python3 urllib
    import urllib.request
except ImportError: # Now importing Python2 urllib
    import urllib

def get_content(url):
    try: # Using Python3 urllib.
        with urllib.request.urlopen(url) as response:
            return response.read() # Returns http.client.HTTPResponse.
    except AttributeError: # Using Python3 urllib.
        return urllib.urlopen(url).read() # Returns an instance.

def norvig_bigtxt():
    url = "https://norvig.com/big.txt"
    # Check if the Norvig's big.txt file exists.
    if os.path.isfile('big.txt'):
        with open('big.txt') as fin:
            return fin.read()
    else: # Otherwise, download the big.txt.
        big_txt = get_content(url).decode('utf8')
        with open('big.txt', 'w') as fout:
            fout.write(big_txt)
        return big_txt



# -*- coding: utf-8 -*-

from __future__ import print_function
import re
import sys

palette = {
    'black': '\033[30m',
    'red': '\033[91m',
    'green': '\033[92m',
    'yellow': '\033[93m',
    'blue': '\033[94m',
    'pink': '\033[95m', 'magenta': '\033[95m',
    'cyan': '\033[96m',
    'white': '\033[97m',
    'gray': '\033[37m',
    'default': '\033[0m',
}

highlighter = {
    'black': '\033[40m',
    'red': '\033[101m',
    'green': '\033[102m',
    'yellow': '\033[103m',
    'blue': '\033[104m',
    'pink': '\033[105m', 'magenta': '\033[105m',
    'cyan': '\033[106m',
    'white': '\033[107m',
    'gray': '\033[47m',
}

formatter = {
    'default': '\033[0m',
    'bold': '\033[1m',
    'faint': '\033[2m',
    'italic': '\033[3m',        # Doesn't work on Ubuntu/Mac terminal.
    'underline': '\033[4m',
    'blinking': '\033[5m',
    'fast_blinking': '\033[6m', # Doesn't work on Ubuntu/Mac terminal.
    'reverse': '\033[7m',       # Note: This reverses the back-/foreground color.
    'hide': '\033[8m',
    'strikethrough': '\033[9m', # Doesn't work on Ubuntu/Mac terminal.
}

def deduplicate(s, ch):
    """
    From http://stackoverflow.com/q/42216559/610569

        s = 'this is   an   irritating string with  random spacing  .'
        deduplicate(s)
        'this is an irritating string with random spacing .'
    """
    return ch.join([substring for substring in s.strip().split(ch) if substring])


def remove_html_tags(s, pad=' '):
    """
    From http://stackoverflow.com/a/12982689/610569
    """
    return re.sub('<.*?>', pad, s)


def rstrip_digit(s):
    """
    From http://stackoverflow.com/a/40691501/610569
    """
    return re.sub(r'\d+$', '', s)


def remove_text_inside_brackets(s, brackets="()[]"):
    """
    From http://stackoverflow.com/a/14603508/610569
    """
    count = [0] * (len(brackets) // 2) # count open/close brackets
    saved_chars = []
    for character in s:
        for i, b in enumerate(brackets):
            if character == b: # found bracket
                kind, is_close = divmod(i, 2)
                count[kind] += (-1)**is_close # `+1`: open, `-1`: close
                if count[kind] < 0: # unbalanced bracket
                    count[kind] = 0
                break
        else: # character is not a bracket
            if not any(count): # outside brackets
                saved_chars.append(character)
    return ''.join(saved_chars)


def color_print(s, color=None, highlight=None, end='\n', file=sys.stdout,
                **kwargs):
    """
    From http://stackoverflow.com/a/287944/610569
    See also https://gist.github.com/Sheljohn/68ca3be74139f66dbc6127784f638920
    """
    if color in palette and color != 'default':
        s = palette[color] + s
    # Highlight / Background color.
    if highlight and highlight in highlighter:
        s = highlighter[highlight] + s
    # Custom string format.
    for name, value in kwargs.items():
        if name in formatter and value == True:
            s = formatter[name] + s
    print(s + palette['default'], end=end, file=file)


def color_str(s, color=None, highlight=None, **kwargs):
    if color in palette and color != 'default':
        s = palette[color] + s
    # Highlight / Background color.
    if highlight and highlight in highlighter:
        s = highlighter[highlight] + s
    # Custom string format.
    for name, value in kwargs.items():
        if name in formatter and value == True:
            s = formatter[name] + s
    return s + palette['default']



# -*- coding: utf-8 -*-

import fnmatch
import glob
import os
import sys


def find_files(dir_path, extension="*"):
    """
    From https://stackoverflow.com/a/2186565/610569
    """
    if sys.version_info.major == 3 and sys.version_info.minor >= 5:
        pattern = '/'.join([dir_path, '**', extension])
        for filename in glob.iglob(pattern, recursive=True):
            yield filename
    else:
        for root, dirnames, filenames in os.walk(dir_path):
            for filename in fnmatch.filter(filenames, extension):
                yield os.path.join(root, filename)



# -*- coding: utf-8 -*-

from __future__ import print_function

import io
import os
import sys
import itertools
from time import gmtime, strftime
from zipfile import ZipFile

try: # Try to use a faster json library.
    import ujson as json
except ImportError: # Otherwise, fall back on native json.
    import json

def iter_paragraph(filename, filetype):
    """
    A helper function to iterate through the diff types of Wikipedia data inputs.
    :param arguments: The docopt arguments
    :type arguments: dict
    :return: A generator yielding a pargraph of text for each iteration.
    """
    assert filetype in ['jsonzip', 'jsondir', 'wikidump']

    # Iterating through paragraphes from the Anntoated Wikipedia zipfile.
    if filetype == 'jsonzip':
        with ZipFile(filename, 'r') as zip_in:
            # Iterate through the individual files.
            for infile in zip_in.namelist():
                if infile.endswith('/'): # Skip the directories.
                    continue
                print(infile, end='\n', file=sys.stderr) # Logging progress.
                with zip_in.open(infile) as f_in:
                    for line in io.TextIOWrapper(f_in, 'utf8'):
                        # Each line is a separate json.
                        data = json.loads(line)
                        # The useful text under 'text' key.
                        yield data['text'].strip()

    # Iterating through paragraphes from the Anntoated Wikipedia directory.
    elif filetype == 'jsondir':
        for root, dirs, files in os.walk(filename):
            for wiki_file in files:
                infile = os.path.join(root, wiki_file)
                print(infile, end='\n', file=sys.stderr) # Logging progress.
                with io.open(infile, 'r', encoding='utf8') as f_in:
                    for line in f_in:
                        # Each line is a separate json.
                        data = json.loads(line)
                        # The useful text under 'text' key.
                        yield data['text'].strip()

    # Iterating through paragraphes from the Wikipedia dump.
    elif filetype == 'wikidump':
        # Simply iterate through every line in the dump
        # and treat each line as a paragraph.
        with io.open(filename, 'r', encoding='utf8') as f_in:
            for line_count, paragraph in enumerate(f_in):
                if line_count % 100000:
                    _msg = 'Processing line {}\n'.format(line_count)
                    print(_msg, file=sys.stderr) # Logging progress.
                if pargraph:
                    yield paragraph





def parallelize_preprocess(func, iterator, processes, progress_bar=False):
    from joblib import Parallel, delayed
    from tqdm import tqdm
    
    iterator = tqdm(iterator) if progress_bar else iterator
    return Parallel(n_jobs=processes)(delayed(func)(line) for line in iterator)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014-17 alvations
# URL:
# For license information, see LICENSE.md

from distutils.core import setup

setup(
    name='lazyme',
    version='0.0.22',
    packages=['lazyme',],
    url = 'https://github.com/alvations/lazyme',
    description='Lazy python recipes',
    license="MIT",
)




from lazyme.fileio import find_files

from lazyme.iterate import per_section, per_chunk, per_window
from lazyme.iterate import zigzag

from lazyme.string import color_print, color_str
from lazyme.string import deduplicate
from lazyme.string import remove_text_inside_brackets, remove_html_tags

from lazyme.wikipedia import iter_paragraph as iter_wiki

from lazyme.data import get_content, norvig_bigtxt
